import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.xml.parsers.*;

import org.w3c.dom.*;

import com.mindfusion.drawing.*;
import com.mindfusion.diagramming.*;
import com.mindfusion.diagramming.Shape;


public class TreeMapFrame extends JFrame
{
	private Diagram diagram;
	private JPanel pnlButtonsPanel;
	private JRadioButton rbtnWeighByPopulation;
	private JRadioButton rbtnWeighByArea;
	private JRadioButton rbtnColorByPopulation;
	private JRadioButton rbtnColorByArea;
	private Color[] scale;
	private final float coef = 3;
	
	public TreeMapFrame()
	{
		super("MindFusion.Diagramming Sample: Tree Map");

		scale = new Color[]{
			Color.decode("#20C300"),
			Color.decode("#FFFF00"),
			Color.decode("#FF8000")
		};
		
		
		this.setBounds(0, 0, 1000, 600);
		this.setVisible(true);
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		
		diagram = new Diagram();
		diagram.setBounds(new Rectangle2D.Float(0,0,1000,600));
		diagram.setMeasureUnit(GraphicsUnit.Millimeter);
		diagram.setDefaultShape(Shape.fromId("Rectangle"));
		diagram.setBackBrush(new SolidBrush(Color.white));
		
		DiagramView view = new DiagramView();
		view.setDiagram(diagram);
		JScrollPane scrPane = new JScrollPane(view);
		scrPane.setVisible(true);
		
		rbtnColorByArea = new JRadioButton("Area");
		rbtnColorByPopulation = new JRadioButton("Population");
		rbtnColorByArea.setSelected(true);
		
		rbtnWeighByArea = new JRadioButton("Area");
		rbtnWeighByPopulation = new JRadioButton("Population");
		rbtnWeighByArea.setSelected(true);
		
		addListeners();
		
		ButtonGroup byColor = new ButtonGroup();
		byColor.add(rbtnColorByArea);
		byColor.add(rbtnColorByPopulation);
		
		ButtonGroup byWeigh = new ButtonGroup();
		byWeigh.add(rbtnWeighByArea);
		byWeigh.add(rbtnWeighByPopulation);
		
		JLabel lblByColor = new JLabel("Color By: ");
		JLabel lblByWeigh = new JLabel("Weigh By: ");
		
		pnlButtonsPanel = new JPanel();
		pnlButtonsPanel.setLayout(new FlowLayout());
		pnlButtonsPanel.setBounds(0, 520, 200,50);
		pnlButtonsPanel.setVisible(true);
		
		pnlButtonsPanel.add(lblByColor);
		pnlButtonsPanel.add(rbtnColorByArea);
		pnlButtonsPanel.add(rbtnColorByPopulation);
		
		pnlButtonsPanel.add(lblByWeigh);
		pnlButtonsPanel.add(rbtnWeighByArea);
		pnlButtonsPanel.add(rbtnWeighByPopulation);
				
		view.setDiagram(diagram);
		view.setBounds(0,0,1395, 1110);
		view.setVisible(true);
		
		this.setLayout(new BorderLayout());
		this.add(scrPane, BorderLayout.CENTER);
		this.add(pnlButtonsPanel, BorderLayout.PAGE_END);
		
		createTreeMap();
		assignWeights();
		assignColors();
		arrangeDiagram();
		
		repaint();
	}
	
	private void createTreeMap()
	{
		diagram.clearAll();

		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = null;
		Document doc = null;
		try
		{
			db = dbf.newDocumentBuilder();
			File file = new File("Data.xml");		
			doc = db.parse(file);
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		Element continentsNode = (Element)doc.getDocumentElement();
		NodeList continents = continentsNode.getElementsByTagName("continent");
		for (int i = 0; i<continents.getLength(); i++)
		{
			Element continent = (Element)continents.item(i);
			ContainerNode container = diagram.getFactory().createContainerNode(0,0,10,10);
			container.setTag(continent);
			container.setCaption(continent.getAttribute("name"));
			
			Element countriesNode = (Element)continent.getElementsByTagName("countries").item(0);
			NodeList countries = countriesNode.getElementsByTagName("country");
			for (int j = 0; j<countries.getLength(); j++)
			{
				Element country = (Element)countries.item(j);
				ShapeNode shape = diagram.getFactory().createShapeNode(0,0,10,10);
				shape.setTag(country);
				shape.setText(country.getAttribute("name"));
				
				double area = Double.parseDouble(country.getAttribute("area"));
				double population = Double.parseDouble(country.getAttribute("population"));
				String areaQuantifier = "million sq. km.";
				if (area<1)
				{
					area*=1000;
					areaQuantifier = "thousand sq. km.";
				}
				String populationQuantifier = "billions";
				if (population<1)
				{
					population *= 1000;
					populationQuantifier = "millions";
				}
				
				shape.setToolTip(String.format("%s\nArea: ~%f %s\nPopulation: ~%f %s.",shape.getText(), area, areaQuantifier, population, populationQuantifier));
				container.add(shape);
			}
		}
	}
	private void assignWeights()
	{
		for (DiagramNode node : diagram.getNodes())
		{
			// Ignore containers and container captions
			if (node instanceof ContainerNode || node.getLocked())
				continue;

			if (node instanceof ShapeNode)
			{
				if (node.getTag() instanceof Element)
				{
					Element element = (Element)node.getTag();
					if (rbtnWeighByArea.isSelected())
						node.setWeight(Float.parseFloat(element.getAttribute("area")));
					if (rbtnWeighByPopulation.isSelected())
						node.setWeight(Float.parseFloat(element.getAttribute("population")));
				}
				else
				{
					continue;
				}
			}
		}
	}
	
	private void assignColors()
	{
		HashMap<ContainerNode, Double> largestArea = new HashMap<ContainerNode, Double>();
		HashMap<ContainerNode, Double> largestPopulation = new HashMap<ContainerNode, Double>();
		for (DiagramNode node : diagram.getNodes())
		{
			if (node instanceof ContainerNode)
			{
				ContainerNode container = (ContainerNode)node;
				double area = 0.0;
				double population = 0.0;
				for (DiagramNode child : container.getChildren())
				{
					if (child.getTag() instanceof Element)
					{
						Element element = (Element)child.getTag();
						area = Math.max(area, Double.parseDouble(element.getAttribute("area")));
						population = Math.max(population, Double.parseDouble(element.getAttribute("population")));
					}
					else continue;
				}
				largestArea.put(container, area);
				largestPopulation.put(container, population);
			}
		}
		for (DiagramNode node:diagram.getNodes())
		{
			// Ignore containers and container captions
			if (node instanceof ContainerNode || node.getLocked())
				continue;

			if (node instanceof ShapeNode)
			{
				if (node.getTag() instanceof Element)
				{
					Element element = (Element)node.getTag();
					double value = 0;
					double total = 0;
					if (rbtnColorByArea.isSelected())
					{
						value = Double.parseDouble(element.getAttribute("area"));
						total = largestArea.get(node.getContainer());
					}
					if (rbtnColorByPopulation.isSelected())
					{
						value = Double.parseDouble(element.getAttribute("population"));
						total = largestPopulation.get(node.getContainer());
					}
					node.setBrush(new SolidBrush(getScaledColor(value, total)));
				}
			}
		}
	}
	
	private Color getScaledColor(double value, double total)
	{
		double factor = value/total;
		if (factor<0)
			factor = 0;
		if (factor>1)
			factor = 1;
		
		double step = 1.0/(scale.length - 1);
		Color start = scale[(int)(factor/step)];
		Color end = scale[Math.min(scale.length-1, (int)(factor/step)+1)];
		
		double factorBase = factor - factor % step;
		return new Color(
				(start.getRed()+(int)((factor-factorBase)*(end.getRed()-start.getRed())/step))%256,
				(start.getGreen()+(int)((factor-factorBase)*(end.getGreen()-start.getGreen())/step))%256,
				(start.getBlue()+(int)((factor-factorBase)*(end.getBlue()-start.getBlue())/step))%256
		);
	}
	
	private void arrangeDiagram()
	{
		TreeMapLayout layout = new TreeMapLayout();
		layout.setPadding(0);
		layout.setContainerPadding(4);
		layout.setLayoutArea(new Rectangle2D.Float(0,0,900/coef,540/coef));
		layout.arrange(diagram);

		// Remove the text of smaller nodes
		for (DiagramNode node : diagram.getNodes())
		{
			if (node instanceof ShapeNode)
			{
				ShapeNode shape = (ShapeNode)node;
				if (node.getBounds().width < 25/coef || node.getBounds().height < 25/coef)
					shape.setText("");
			}
		}

		// Resize caption nodes
		for (DiagramNode node : diagram.getNodes())
		{
			if (node.getLocked() && node.getContainer() != null)
			{
				Rectangle2D.Float bounds = node.getBounds();
				bounds.width = node.getContainer().getBounds().width;
				bounds.height = Math.min(30, node.getContainer().getBounds().height);
				node.setBounds(bounds);
			}
		}
	}

	private void onOptionChanged()
	{
		if (diagram == null)
			return;

		// Reassign weights, rearrange, and recolor
		assignWeights();
		assignColors();
		arrangeDiagram();
	}

	private void addListeners()
	{
		rbtnColorByArea.addActionListener(new ActionListener(){

			@Override
			public void actionPerformed(ActionEvent e)
			{
				onOptionChanged();
			}
			
		});
		rbtnColorByPopulation.addActionListener(new ActionListener(){

			@Override
			public void actionPerformed(ActionEvent e)
			{
				onOptionChanged();
			}
			
		});
		rbtnWeighByArea.addActionListener(new ActionListener(){

			@Override
			public void actionPerformed(ActionEvent e)
			{
				onOptionChanged();
			}
			
		});
		rbtnWeighByPopulation.addActionListener(new ActionListener(){

			@Override
			public void actionPerformed(ActionEvent e)
			{
				onOptionChanged();
			}
			
		});
	}
		
	private static final long serialVersionUID = 1L;
}